@@ -277,17 +277,24 @@ static int
 image_new_from_file(const char *filename)
 {
     Imlib_Image imimage;
+    Imlib_Load_Error e = IMLIB_LOAD_ERROR_NONE;
     image_t *image;
 
     if(!filename)
         return 0;
 
-    if(!(imimage = imlib_load_image_without_cache(filename)))
+    if(!(imimage = imlib_load_image_with_error_return(filename, &e)))
     {
-        warn("cannot load image %s", filename);
+        warn("cannot load image %s: %s", filename, image_imlib_load_strerror(e));
         return 0;
     }
 
+    /* Make imlib check if the file changed on disk if it's later opened by the
+     * same file name again before using its cache.
+     */
+    imlib_context_set_image(imimage);
+    imlib_image_set_changes_on_disk();
+
     image = image_new(globalconf.L);
     image->image = imimage;
 
